@@ -1,4 +1,4 @@
-from mhkit.river import device 
+from mhkit.river import performance 
 from mhkit.river import graphics
 from mhkit.river import io
 from mhkit.river import resource 
